declare global {
   interface Window {
      requestIdleCallback: (callback: (deadline: { didTimeout: boolean, timeRemaining: () => number }) => void, options?: { timeout: number }) => number | NodeJS.Timeout;
      cancelIdleCallback: (id: number | NodeJS.Timeout) => void;
      debug: any;
   }

   namespace JSX {
      interface IntrinsicElements {
         "wired-button": HTMLAttributes;
         "wired-card": HTMLAttributes;
         "wired-checkbox": HTMLAttributes;
         "wired-combo": HTMLAttributes;
         "wired-fab": HTMLAttributes;
         "wired-icon-button": HTMLAttributes;
         "wired-input": HTMLAttributes;
         "wired-item": HTMLAttributes;
         "wired-lib": HTMLAttributes;
         "wired-listbox": HTMLAttributes;
         "wired-progress": HTMLAttributes;
         "wired-radio-group": HTMLAttributes;
         "wired-radio": HTMLAttributes;
         "wired-slider": HTMLAttributes;
         "wired-spinner": HTMLAttributes;
         "wired-tabs": HTMLAttributes;
         "wired-textarea": HTMLAttributes;
         "wired-toggle": HTMLAttributes;
         "wired-tooltip": HTMLAttributes;

      }
   }
}
// declare const window: Window;

window.requestIdleCallback =
   window.requestIdleCallback ||
   function (cb) {
      var start = Date.now();
      return setTimeout(function () {
         console.log("Idle Timeout reached!");
         cb({
            didTimeout: false,
            timeRemaining: function () {
               return Math.max(0, 50 - (Date.now() - start));
            }
         });
      }, 1);
   };

window.cancelIdleCallback =
   window.cancelIdleCallback ||
   function (id) {
      clearTimeout(id as any);
   };

console.log(window.requestIdleCallback);

window.debug = {};




import { h, render } from 'preact';
import App from './components/App';


import "uikit";
import "uikit/dist/css/uikit.css"

import "./index.scss";
import Navigation from './navigation';
import VaultsPage from './components/routes/vaults/Vaults';
import { Page } from './page';

import Notes from "./notes"
import DemoPage from './components/demo';
import VaultPage from './components/routes/vault/Vault';
import SharePage from './components/routes/share/Share';
import Notifications from './notifications';
window.debug.notes = Notes;

(async () => {
   // Initialize notes provider
   if (Notes.loginRequired()) {
      let url = new URL(location.href)
      let code = url.searchParams.get("code");

      if (code) {
         let err = await Notes.getToken(code)
         if (err) {
            Notifications.sendError("Login failed: " + err)
            Notes.login()
         } else {
            window.history.replaceState(null, document.title, "/" + window.location.hash);
         }
      } else {
         Notes.login()
      }
   }

   await Notes.start();

   Navigation.default = VaultsPage as typeof Page;
   Navigation.addPage("/vault", VaultPage as typeof Page)
   Navigation.addPage("/demo", DemoPage as typeof Page)
   Navigation.addPage("/share", SharePage as typeof Page)
   Navigation.start();

   render(<App />, document.body, document.getElementById('app'));
})()
